package org.jabber;

import java.net.*;
import java.io.*;
import java.util.*;

/** A <code>Channel</code> represents a connection between a local client
 * and a Jabber transport.  The channel may be connected or disconnected
 * at any given time.
 *
 * A <code>Channel</code> is a JavaBean.  In its serialized form, the 
 * socket will be lost, but all of the connection parameters will
 * persist.  Upon deserialization, the channel will be disconnected.
 *
 * A <code>Channel</code> never attempts to auto-connect.  If you want
 * auto-connection, you should create a ConnectionListener to catch
 * disconnect events and tell the channel to reconnect.
 *
 * @author Mike Nygard
 * @version 0.01
 */
public class Channel implements Serializable {
	public static final String DEFAULT_TRANSPORT_HOST = "localhost";
	public static final int DEFAULT_TRANSPORT_PORT = 5222;

	private Vector connectionListeners = new Vector();
	private String transportHost = DEFAULT_TRANSPORT_HOST;
	private int transportPort = DEFAULT_TRANSPORT_PORT;
	private ChannelState state = null;

	abstract class Invoker {
	abstract void invoke(ConnectionListener l);
	}

	public Channel() {
	ChannelDisconnected dis = new ChannelDisconnected();
	dis.setChannel(this);
	state = dis;
	}
	public Channel(Socket socket) throws ConnectionException {
	ChannelDisconnected dis = new ChannelDisconnected();
	setState(dis);
	dis.buildConnection(socket);
	}
	//
	// Event Source: ConnectionEvent
	//
	public synchronized void addConnectionListener(ConnectionListener l) {
	if(!connectionListeners.contains(l))
	    connectionListeners.addElement(l);
	}
	/** Connect to a transport host.  If the channel is already connected,
	 * this method returns without doing anything.
	 *<br>
	 * ConnectionListeners will be notified when the channel starts
	 * trying to connect, and when the attempt finishes, either
	 * successfully or unsuccessfully.
	 *
	 * @exception UnknownHostException if the IP address of the 
	 *            transportHost cannot be resolved.
	 */
	public synchronized void connect() throws UnknownHostException {
	try {
	    if(isConnected()) {
		return;
	    }

	    InetAddress host = InetAddress.getByName(getTransportHost());

	    fireAttempting(host);

	    // Delegate the meat of the connection attempt.
	    getState().connect(host, getTransportPort());

	    fireConnected(host);
	} catch(ConnectionException ce) {
	    fireConnectFailed(ce);
	}
	}
	public synchronized void disconnect() throws ConnectionException {
	getState().disconnect();
	}
	//
	// Convenience methods for event dispatching
	//
	protected void fireAttempting(InetAddress host) {
	final ConnectionEvent evt = new ConnectionEvent(this, host);
	
	fireConnectionEvent(new Invoker() {
	    public void invoke(ConnectionListener l) {
		l.attemptingConnection(evt);
	    }
	});
	}
	protected void fireConnected(InetAddress host) {
	final ConnectionEvent evt = new ConnectionEvent(this, host);
	    
	fireConnectionEvent(new Invoker() {
	    public void invoke(ConnectionListener l) {
		l.channelConnected(evt);
	    }
	});
	}
	protected void fireConnectFailed(Exception exc) {
	final ConnectionEvent evt = new ConnectionEvent(this, exc);

	fireConnectionEvent(new Invoker() {
	    public void invoke(ConnectionListener l) {
		l.connectFailed(evt);
	    }
	});
	}
	private void fireConnectionEvent(Invoker invoker) {
	Vector targets;

	synchronized(this) {
	    targets = (Vector)connectionListeners.clone();
	}


	for(Enumeration e = targets.elements(); e.hasMoreElements(); ) {
	    invoker.invoke((ConnectionListener)e.nextElement());
	}
	}
	protected void fireDisconnected(String reason) {
	final ConnectionEvent evt = new ConnectionEvent(this, reason);

	fireConnectionEvent(new Invoker() {
	    public void invoke(ConnectionListener l) {
		l.channelDisconnected(evt);
	    }
	});
	}
	//
	// State management
	//
	protected ChannelState getState() {
	return this.state;
	}
	public String getTransportHost() {
	return transportHost;
	}
	public int getTransportPort() {
	return this.transportPort;
	}
	public boolean isConnected() {
	return getState().isConnected();
	}
	public synchronized void removeConnectionListener(ConnectionListener l) {
	connectionListeners.removeElement(l);
	}
	protected void setState(ChannelState state) {
	this.state = state;
	state.setChannel(this);
	}
	//
	// Properties
	//
	public void setTransportHost(String hostname)  {
	this.transportHost = hostname;
	}
	public void setTransportPort(int port) {
	this.transportPort = port;
	}
}