#!/usr/local/bin/perl
# 
# Jabber client
# jeremie@jabber.org

package Net::Jabber;

=head1 NAME

Net::Jabber - Jabber Client Library

=head1 SYNOPSIS

    use Net::Jabber;

    # define your callback functions for incoming data
    SetCallbacks("connect" => \&connect, "message" => \&message, "status" => \&status);

    # connect to server, defaults to localhost if no server specified
    Connect("jabber.server");

    # hand off all processing to module
    Process();

    # alternatively, you can be at the realm by passing a timeout in seconds
    # you must call this frequently to check for incoming data
    Process(2);

=head1 DESCRIPTION

Net::Jabber description coming soon.

=head1 AUTHOR

By Jeremie Miller in May of 1999 for http://jabber.org/

=head1 COPYRIGHT

This module is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut


BEGIN
{
	require 5.003;
	use strict;
	use IO::Socket;
	use IO::Select;
	use XML::Parser;
	use Carp;
	use vars qw($VERSION $PROTOCOL @ISA @EXPORT @EXPORT_OK);
	use vars qw($PROTOCOL %SERVER %CB %PACKET $PACKET_NAME $TAG);
	require Exporter;
	@ISA = qw(Exporter);
	@EXPORT = qw(&SetCallbacks &Connect &Process &SendSimple &SendStatus &SendRoster &Simplify &EscapeArray);
	@EXPORT_OK = qw();

	die "XML::Parser must be 2.22 or newer" unless $XML::Parser::Expat::VERSION > 2.22;

	$VERSION = '0.01';
	$PROTOCOL = '19990505';
	%SERVER = ("Name" => "127.0.0.1", "Port" => 5222, "Sock" => 0);
	%CB = ("connect" => \&_connect, "message" => \&_message, "status" => \&_status, "roster" => \&_roster);
}


##############
# 
# Public Methods
#

sub SetCallbacks
{
	my %arg = @_;
	map($CB{$_} = $arg{$_}, keys(%arg));
}

sub Connect
{
	$SERVER{'Name'} = $_[0] if($_[0]);
	$SERVER{'Sock'} = new IO::Socket::INET(PeerAddr => $SERVER{'Name'}, PeerPort => $SERVER{'Port'}, Proto => 'tcp');
	return undef unless $SERVER{'Sock'};
	$SERVER{'Sock'}->autoflush(1);

	$SERVER{'Sock'}->print(<<EOF) || return undef;
<?xml version="1.0"?>
<jabber version="Net::Jabber/$VERSION" protocol="$PROTOCOL">
EOF

	my $expat = new XML::Parser(Handlers => {Start => \&_handle_hello, End   => \&_handle_close, Char  => \&_handle_cdata});
	$SERVER{'Parser'} = $expat->parse_start();
	$SERVER{'Select'} = new IO::Select($SERVER{'Sock'});

	1;
}

sub Process
{
	my $timeout = $_[0];
	my ($buff);

	return undef unless($SERVER{'Select'} || $SERVER{'Parser'});

	# if $timeout is given and nothing is found, it returns, 
	# else it blocks and loops forever unless error
	while($SERVER{'Select'}->can_read($timeout))
	{
			$SERVER{'Sock'}->sysread($buff,1024);
			$SERVER{'Parser'}->parse_more($buff);
			return undef if($SERVER{'Error'});
	}
	1;
}

sub SendSimple
{
	my ($tag, %data) = EscapeArray(@_);
	my $str = "<$tag>";
	map(($str .= "<$_>$data{$_}</$_>"), keys(%data));
	$str .= "</$tag>";
	$SERVER{'Sock'}->print($str) || ($SERVER{'Error'} = 1);
}

sub SendStatus
{
	my ($say,$icon,$type,$ext) = EscapeArray(@_);
	$type = (length($type) > 0)?" type='$type'":"";
	my $str = "<status$type><say>$say</say>";
	$str .= "<icon>$icon</icon>" if(length($icon) > 0);
	$str .= "<ext>$ext</ext>" if(length($ext) > 0);
	$str .= "</status>";
	$SERVER{'Sock'}->print($str) || ($SERVER{'Error'} = 1);
}

sub SendRoster
{
	my ($cmd,$group,$user) = EscapeArray(@_);
	$group = (length($group) > 0)?" group='$group'":"";
	my $str = "<roster><$cmd$group>$user</$cmd></roster>";
	$SERVER{'Sock'}->print($str) || ($SERVER{'Error'} = 1);
}

sub EscapeArray
{
	my(@in) = @_;
	foreach(@in)
	{
		$_ =~ s/&/&amp;/g;
		$_ =~ s/</&lt;/g;
		$_ =~ s/>/&gt;/g;
		$_ =~ s/"/&quot;/g;
		$_ =~ s/'/&apos;/g;
	}
	return @in;
}

sub Simplify
{
	my(%tag) = @_;
	return () if($tag{'name'} eq "ext"); #ignore <ext></ext> packets

	my(@ret) = ($tag{'name'},$tag{'data'});
	my(@att) = %{$tag{"attribs"}};
	push(@ret,@att) if($tag{"attribs"});

	if($tag{'children'})
	{
		foreach(@{$tag{"children"}})
		{
			push(@ret,&Simplify(%{$_}));
		}
	}

	return @ret;
}


##############
# 
# Internal Handlers for XML::Parser
#

sub _handle_hello
{
	# This handles the <jabber> root tag and sets up the packet handler
	my ($expat, $tag, %att) = @_;
	$SERVER{'Error'} = 1 if($tag ne "jabber");
	$SERVER{'Version'} = $att{'version'};
	$SERVER{'Protocol'} = $att{'protocol'};
	$expat->setHandlers(Start => \&_handle_tag);
	&{$CB{"connect"}}(%SERVER) if($CB{"connect"});
	$TAG = "top";
}

sub _handle_tag
{
	my ($expat, $tag, %att) = @_;
	my (%NEW);

	$NEW{"name"} = $tag;
	$NEW{"attribs"} = \%att;
	$NEW{"data"} = "";
	$NEW{"parent"} = $TAG;
	push(@{$TAG->{"children"}},\%NEW);
	$TAG = \%NEW;
}

sub _handle_cdata
{
	my ($expat, $cdata) = @_;
	$TAG->{"data"} .= $cdata;
}

sub _handle_close
{
	my ($expat, $tag) = @_;
	if($TAG->{"parent"} eq "top")
	{
		&{$CB{$tag}}(%{$TAG}) if($CB{$tag});
	}
	$TAG = $TAG->{"parent"};
}



##############
# 
# Internal Default Callbacks(for debugging only!)
#

sub _printXML
{
	my %arg = @_;
	print "<$arg{'name'}";
	my %att = %{$arg{"attribs"}};
	map(print(" $_='$att{$_}'"),keys(%att));
	print ">";
	if($arg{"children"})
	{
		map(_printXML(%{$_}),@{$arg{"children"}});
	}else{
		print $arg{"data"};
	}
	print "</$arg{'name'}>\n";
}

sub _connect
{
	my %arg = Simplify(@_);
	print "Got connect\n";
	map((print "\t$_:$arg{$_}\n"), keys(%arg));
}

sub _message
{
	my %arg  = Simplify(@_);
	print "Got message\n";
	map((print "\t$_:$arg{$_}\n"), keys(%arg));
}

sub _status
{
	my %arg = Simplify(@_);
	print "Got status\n";
	map((print "\t$_:$arg{$_}\n"), keys(%arg));
}

sub _roster
{
	my %arg = Simplify(@_);
	print "Got roster\n";
	map((print "\t$_:$arg{$_}\n"), keys(%arg));
}


END { }

1;


