
#include "element.h"

#include <stdlib.h>

element element_new(const char* name, unsigned short type)
{
     element result = NULL;
     /* Ensure type is valid */
     if (type > ETYPE_LAST)
	  return NULL;
     /* Allocate result memory */
     result = (element)malloc(sizeof(_element));
     /* Zero out structure */
     memset(result, '\0', sizeof(_element));
     /* Initialize fields */
     if (name != NULL)
	  result->name = (void*)strdup(name);
     result->type = type;
     result->next = result;
     result->prev = result;
     return result;
}

element element_new_child(element parent, const char* name, unsigned short type)
{
     /* Ensure type & parent is valid */
     if ((type > ETYPE_LAST)||(parent == NULL))
	  return NULL;
     /* If there are no existing children, allocate
	a new one to start the list */
     if (parent->firstchild == NULL) 
	  parent->firstchild = element_new(name, type);
     /* Otherwise, append new sibling to the end of the list */
     else
	  element_new_sibling(parent->prev, name, type);
     /* Return newly created element */
     return parent->firstchild->prev;
}

element element_new_sibling(element sibling, const char* name, unsigned short type)
{
     /* Ensure type & sibling is valid */
     if ((type > ETYPE_LAST)||(sibling == NULL))
	  return NULL;
     /* Allocate new element */
     sibling->next->prev = element_new(name, type);
     sibling->next->prev->next = sibling->next;
     sibling->next->prev->prev = sibling;
     sibling->next = sibling->next->prev;
     return sibling->next;
}

element element_put_attrib(element e, unsigned short fast, const char* name, const char* value)
{
     element a; 
     /* Ensure e is valid */
     if (e == NULL)
	  return e;
     /* If there are no existing attributes, allocate
	a new one to start the list */
     if (e->attribs == NULL)
     {
	  e->attribs = element_new(name, ETYPE_ATTRIB);
	  e->attribs->data = (void*)strdup(value);
	  return e->attribs;
     }
     /* Else search for an existing attribute with
	that name, if fast isn't set  */
     if ((fast == 0) && ((a = element_get_attrib(e, name)) != NULL))
     {
	  free(a->data);
	  a->data = (void*) strdup(value);
	  return a;
     }
     /* Else create a new attribute and append it */
     a = e->attribs;
     a->prev = element_new(name, ETYPE_ATTRIB);
     a->next->prev->next = a->next;
     a->next->prev->prev = a;
     a->next = a->next->prev;
     return a->next;
}

element element_get_attrib(element e, const char* name)
{
     element current;
     /* Ensure e && e->attribs are valid */
     if ((e == NULL)||(e->attribs == NULL))
	  return NULL;
     /* Otherwise, iterate across list looking
	for element with specified name */
     current = e->attribs;
     while (current != NULL)
     {
	  /* Compare names and return if match is found */
	  if (strcmp(current->name, name) == 0)
	       return current;
	  /* Traverse to the next element */
	  current = element_next(current);
     }
     return NULL;
}

const char* element_get_attribv(element e, const char* name)
{
     /* Use element_get_attrib to lookup attrib structure */
     element a = element_get_attrib(e, name);
     /* If a != NULL, return the value */
     if (a != NULL)
	  return (char*)a->data;
     return NULL;
}

element element_get_child(element parent, const char* name)
{
     element current;
     /* Ensure parent is not null, and we have a valid
	search parameter */
     if ((parent == NULL))
	  return NULL;
     /* If no name is provided, simply get the first child */
     if (name == NULL)
	  return parent->firstchild;
     /* Begin traversing child list */
     current = parent->firstchild;
     while (current != NULL)
     {
	  /* Check the name */
	  if (strcmp(current->name, name) == 0)
	       return current;
	  /* Continue traversal */
	  current = element_next(current);
     }
     return NULL;
}

element element_next(element e)
{
     /* Ensure e is valid, and e-> next isn't
	just e again. Ensure next stops when
	the end of the list is hit */
     if ((e == NULL)||(e == e->next))
	  return NULL;
     return e->next;
}

element element_previous(element e)
{
     /* Ensure e is valid and e->prev isn't
	just e again. Ensure prev stops when
	the beginning of the list is hit */
     if ((e == NULL)||(e == e->prev))
	  return NULL;
     return e->prev;
}

element element_lastchild(element e)
{
     /* Ensure e && e->firstchild is not NULL */
     if ((e == NULL) || (e->firstchild == NULL))
	  return NULL;
     return e->firstchild->prev;
}

void element_import_attribs(element e, char** attribs)
{
     int i = 0;
     char *n, *v;
     /* Ensure e is not NULL */
     if (e == NULL)
	  return;
     while (attribs[i] != '\0')
     {
	  /* Get name, calculate offset of next string */
	  n = (char*)attribs[i]; i = strlen(n)+2;
	  /* Get value, calculate offset of next string */
	  v = (char*)attribs[i]; i = strlen(v)+2;
	  /* Add the attribute to the element, using the fast flag */
	  element_put_attrib(e,E_FAST,n,v);
     }
}
	  
     
