#ifndef INCL_ELEMENT_H
#define INCL_ELEMENT_H

#define ETYPE_CDATA  0
#define ETYPE_TAG    1
#define ETYPE_ATTRIB 2

#define ETYPE_LAST   2
#define ETYPE_UNDEF  -1

#define E_FAST 1
#define E_THOROUGH 0

/* Element structure. Do not use directly! Always use accessor macros and methods! */

typedef struct element_t
{
     char*           name;
     unsigned short  type;
     void*           data;
     struct element_t*     firstchild;
     struct element_t*     prev;
     struct element_t*     next;
     struct element_t*     attribs;
} _element, *element;

/* element_new, element_new_child, element_new_sibling
   Desc: Creates a new element (at different levels), requires a valid element ID
   Returns: 
    On success: newly allocated element
    On failure: NULL
*/
element element_new(const char* name, unsigned short type);

element element_new_child(element parent,
			  const char* name,
			  unsigned short type);

element element_new_sibling(element sibling,
			    const char* name,
			    unsigned short type);
/* element_free
   Desc: Releases a element and all its children and attributes
*/
void    element_free(element e);

/* element_put_attrib
   Desc: Creates/replaces an attribute on the specified element, set
         the fast arg to E_FAST to disable check for existing elements;
	 useful for building the attrib tree
   Returns:
     On success: the new attribute
     On failure: NULL
*/
element element_put_attrib(element e, unsigned short fast, const char* name, const char* value);

/* element_get_attrib
   Desc: Retrieves an attribute from an element, if it exists 
   Returns:
     On success: the requested element
     On failure: NULL
*/
element element_get_attrib(element e, const char* name);

/* element_get_attribv
   Desc: Retrieves an attribute's value from an element, if it exists
*/   
const char* element_get_attribv(element e, const char* name);

/* element_get_child
   Desc: Retrieves the child with the specified name, or 
         simply returns the first child if name == NULL
   Returns:
     On success: the requested element
     On failure: NULL
*/
element element_get_child(element parent,
			  const char* name);


/* element_next, element_previous
   Desc: Attempts to get next/previous sibling element,
         returns NULL if this is first/last element
   Returns:
     On success: the element
     On failure: NULL
*/
element element_next(element e);

element element_previous(element e);

/* element_lastchild, element_firstchild
   Desc: Attempts to get first/last child of a element
   Returns:
     On success: the element
     On failure: NULL
*/
element element_lastchild(element e);
#define element_firstchild(e) (e->firstchild)

/* element_name, element_data, element_type
   Desc: Accessors to fields in the element structure
*/
#define element_name(e) (e->name)
#define element_data(e) (e->data)
#define element_cdata(e)((char*)e->data)
#define element_type(e) (e->type)

#define element_set_data(e, d) (e->data = d)

/* element_import_attribs
 */
void element_import_attribs(element e, char** attribs);

#endif
